# Generated by Django 4.1.6 on 2023-02-17 08:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('anorganization', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='membership',
            name='attributes',
            field=models.JSONField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='organization',
            name='attributes',
            field=models.JSONField(blank=True, null=True),
        ),
    ]



"""
Copyright (c) 2014-present, aglean Inc.
"""
from django.conf import settings
from django.db import models

from .utils import image_path, image_url


class Organization(models.Model):
    # relationship
    members = models.ManyToManyField(settings.AUTH_USER_MODEL,
                                     through='Membership')

    # detail
    name = models.CharField(max_length=255)
    dn = models.CharField(max_length=255, blank=True)
    attributes = models.JSONField(blank=True)
    image = models.ImageField(
        upload_to=image_path,
        help_text=(
            'Upload file should under size limitation, '
            'with png, jpg or jpeg file extensions.'
        ),
        blank=True
    )

    def __str__(self):
        return self.name

    @property
    def image_url(self):
        return image_url(self)

    @property
    def valid_member_count(self):
        return self.members.filter(membership__is_valid=True).count()

    @property
    def invalid_member_count(self):
        return self.members.filter(membership__is_valid=False).count()


class Membership(models.Model):
    # relationship
    user = models.ForeignKey(settings.AUTH_USER_MODEL,
                             on_delete=models.CASCADE)
    organization = models.ForeignKey(Organization, on_delete=models.CASCADE)

    # detail
    serial_number = models.CharField(max_length=100, blank=True)
    dn = models.CharField(max_length=255, blank=True)
    attributes = models.JSONField(blank=True)
    image = models.ImageField(
        upload_to=image_path,
        help_text=(
            'Upload file should under size limitation, '
            'with png, jpg or jpeg file extensions.'
        ),
        blank=True
    )

    # flag
    is_valid = models.BooleanField(
        help_text='Designates whether the membership is valid.',
        default=True
    )

    # datetime
    created_at = models.DateTimeField(auto_now=False, auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True, auto_now_add=False)

    def __str__(self):
        return self.user.username

    @property
    def image_url(self):
        return image_url(self)



"""
Copyright (c) 2014-present, aglean Inc.
"""
from ariadne_relay import NodeObjectType

from .models import Organization, Membership
from .bindables import resolve_anorganization_memberships
from .bindables_sync import resolve_anorganization_memberships_sync


anorganization = NodeObjectType('AnOrganization')
anorganization_membership = NodeObjectType('AnOrganizationMembership')


async def resolve_anorganization_instance(id, *_):
    return await Organization.objects.aget(id=id)


async def resolve_anorganization_member_connection(obj, info, connection_args,
                                                   **kwargs):
    """resolve org connection to members"""
    return await resolve_anorganization_memberships(obj.members,
                                                    info,
                                                    connection_args,
                                                    kwargs)


async def resolve_anorganization_membership_instance(id, *_):
    return await Membership.objects.aget(id=id)


def resolve_anorganization_instance_sync(id, *_):
    return Organization.objects.get(id=id)


def resolve_anorganization_member_connection_sync(obj, info, connection_args,
                                                  **kwargs):
    """resolve org connection to members"""
    return resolve_anorganization_memberships_sync(obj.members, info,
                                                   connection_args, kwargs)


def resolve_anorganization_membership_instance_sync(id, *_):
    return Membership.objects.get(id=id)



"""
Copyright (c) 2014-present, aglean Inc.
"""
import operator
from functools import reduce

from ariadne_relay import from_global_id
from asgiref.sync import sync_to_async
from django.contrib.auth import get_user_model
from django.db.models import Q
from django.db.models.query import QuerySet

from .models import Organization, Membership


async def resolve_anorganizations(obj, info, connection_args, **kwargs):
    name = kwargs.get('name', '')
    dn = kwargs.get('dn', '')

    queryset = []
    lookups = []

    if name:
        lookups.append(Q(name__icontains=name))

    if dn:
        lookups.append(Q(name_icontains=dn))

    if lookups:
        queryset = await sync_to_async(obj.filter)(reduce(operator.and_,
                                                          lookups)) \
                if isinstance(obj, QuerySet) \
                else await sync_to_async(Organization.objects.filter)(
                        reduce(operator.and_, lookups))
    else:
        queryset = obj if isinstance(obj, QuerySet) \
                else await sync_to_async(Organization.objects.all)()

    return queryset


async def resolve_users(obj, info, connection_args, **kwargs):
    username = kwargs.get('username', '')
    first_name = kwargs.get('first_name', '')
    last_name = kwargs.get('last_name', '')
    email = kwargs.get('email', '')
    is_active = kwargs.get('is_active', None)

    queryset = []
    lookups = []

    if username:
        lookups.append(Q(username__icontains=username))

    if first_name:
        lookups.append(Q(first_name__icontains=first_name))

    if last_name:
        lookups.append(Q(last_name__icontains=last_name))

    if email:
        lookups.append(Q(email__icontains=email))

    if is_active is not None:
        lookups.append(Q(is_active=is_active))

    if lookups:
        queryset = await sync_to_async(obj.filter)(reduce(operator.and_,
                                                          lookups)) \
                if isinstance(obj, QuerySet) \
                else await sync_to_async(get_user_model().objects.filter)(
                        reduce(operator.and_, lookups))
    else:
        queryset = obj if isinstance(obj, QuerySet) \
                else await sync_to_async(get_user_model().objects.all)()

    return queryset


async def resolve_anorganization_memberships(obj, indo, connection_args,
                                             **kwargs):
    user = kwargs.get('user', '')
    organization = kwargs.get('organization', '')
    serial_number = kwargs.get('serial_number', '')
    dn = kwargs.get('dn', '')
    is_valid = kwargs.get('is_valid', None)

    queryset = []
    lookups = []

    if user:
        _, id = from_global_id(user)
        lookups.append(Q(user=id))

    if organization:
        _, id = from_global_id(organization)
        lookups.append(Q(organization=id))

    if serial_number:
        lookups.append(Q(serial_number__icontains=serial_number))

    if dn:
        lookups.append(Q(name_icontains=dn))

    if is_valid is not None:
        lookups.append(Q(is_valid=is_valid))

    if lookups:
        queryset = await sync_to_async(obj.filter)(reduce(operator.and_,
                                                          lookups)) \
                if isinstance(obj, QuerySet) \
                else await sync_to_async(Membership.objects.filter)(
                        reduce(operator.and_, lookups))
    else:
        queryset = obj if isinstance(obj, QuerySet) \
                else await sync_to_async(Membership.objects.all)()

    return queryset



"""
Copyright (c) 2014-present, aglean Inc.
"""
from ariadne import load_schema_from_path

from .type_defs import (
    anorganization,
    anorganization_membership,
    resolve_anorganization_instance,
    resolve_anorganization_member_connection,
    resolve_anorganization_membership_instance,
    resolve_anorganization_instance_sync,
    resolve_anorganization_member_connection_sync,
    resolve_anorganization_membership_instance_sync
)
from .bindables import (
    resolve_anorganizations,
    resolve_anorganization_memberships
)
from .bindables_sync import (
    resolve_anorganizations_sync,
    resolve_anorganization_memberships_sync
)


anorganization_schema = load_schema_from_path('anorganization/graphqls')

anorganization_bindables = [anorganization, anorganization_membership]


__all__ = [
    'anorganization_schema',
    'anorganization_bindables',
    resolve_anorganization_instance,
    resolve_anorganization_member_connection,
    resolve_anorganization_membership_instance,
    resolve_anorganization_instance_sync,
    resolve_anorganization_member_connection_sync,
    resolve_anorganization_membership_instance_sync,
    resolve_anorganizations,
    resolve_anorganization_memberships,
    resolve_anorganizations_sync,
    resolve_anorganization_memberships_sync
]



"""
Copyright (c) 2014-present, aglean Inc.
"""
import operator
from functools import reduce

from ariadne import convert_kwargs_to_snake_case
from ariadne_relay import NodeObjectType, from_global_id
from django.contrib.auth import get_user_model
from django.db.models import Q
from django.db.models.query import QuerySet

from .models import Organization, Membership


anorganization = NodeObjectType('AnOrganization')


@anorganization.instance_resolver
def resolve_anorganization_instance(id, *_):
    return Organization.objects.get(id=id)


@anorganization.connection('members')
def resolve_anorganization_member_connection(obj, info, **kwargs):
    return resolve_users(obj.members, info, kwargs)


@convert_kwargs_to_snake_case
def resolve_anorganizations(obj, info, **kwargs):
    name = kwargs.get('name', '')
    dn = kwargs.get('dn', '')

    queryset = []
    filters = []

    if name:
        filters.append(Q(name__icontains=name))

    if dn:
        filters.append(Q(name_icontains=dn))

    if filters:
        queryset = obj.filter(reduce(operator.and_, filters)) \
                if isinstance(obj, QuerySet) \
                else Organization.objects.filter(reduce(operator.and_,
                                                        filters))
    else:
        queryset = obj if isinstance(obj, QuerySet) \
                else Organization.objects.all()

    return queryset


@convert_kwargs_to_snake_case
def resolve_users(obj, info, **kwargs):
    username = kwargs.get('username', '')
    first_name = kwargs.get('first_name', '')
    last_name = kwargs.get('last_name', '')
    email = kwargs.get('email', '')
    is_active = kwargs.get('is_active', None)

    queryset = []
    filters = []

    if username:
        filters.append(Q(username__icontains=username))

    if first_name:
        filters.append(Q(first_name__icontains=first_name))

    if last_name:
        filters.append(Q(last_name__icontains=last_name))

    if email:
        filters.append(Q(email__icontains=email))

    if is_active is not None:
        filters.append(Q(is_active=is_active))

    if filters:
        queryset = obj.filter(reduce(operator.and_, filters)) \
                if isinstance(obj, QuerySet) \
                else get_user_model().objects.filter(reduce(operator.and_,
                                                            filters))
    else:
        queryset = obj if isinstance(obj, QuerySet) \
                else get_user_model().objects.all()

    return queryset


anorganization_membership = NodeObjectType('AnOrganizationMembership')


@anorganization_membership.instance_resolver
def resolve_anorganization_membership_instance(id, *_):
    return Membership.objects.get(id=id)


@convert_kwargs_to_snake_case
def resolve_anorganization_memberships(obj, indo, **kwargs):
    user = kwargs.get('user', '')
    organization = kwargs.get('organization', '')
    serial_number = kwargs.get('serial_number', '')
    dn = kwargs.get('dn', '')
    is_valid = kwargs.get('is_valid', None)

    queryset = []
    filters = []

    if user:
        _, id = from_global_id(user)
        filters.append(Q(user=id))

    if organization:
        _, id = from_global_id(organization)
        filters.append(Q(organization=id))

    if serial_number:
        filters.append(Q(serial_number__icontains=serial_number))

    if dn:
        filters.append(Q(name_icontains=dn))

    if is_valid is not None:
        filters.append(Q(is_valid=is_valid))

    if filters:
        queryset = obj.filter(reduce(operator.and_, filters)) \
                if isinstance(obj, QuerySet) \
                else Membership.objects.filter(reduce(operator.and_, filters))
    else:
        queryset = obj if isinstance(obj, QuerySet) \
                else Membership.objects.all()

    return queryset


types = [anorganization, anorganization_membership]
